import junit.framework.Assert;
import org.junit.Test;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import static org.junit.Assert.*;

public class CheckbookTest {
    Checkbook checkbook;
    Transaction transaction;
    Payee payee1;
    Transaction transaction2;
    Payee payee2;
    Transaction transaction3;
    Payee payee3;
    Transaction transaction4;
    Payee payee4;


    @org.junit.Before
    public void setUp() throws Exception {
        checkbook = new Checkbook(20000);
        payee1 = new Payee();
        payee2 = new Payee();
        payee3 = new Payee();
        payee3 = new Payee();
        transaction = new Transaction("one",payee1, TransactionType.DEBIT, 200.0);
        transaction2 = new Transaction("two",payee2, TransactionType.CREDIT, 580.0);
        transaction3 = new Transaction("three",payee3, TransactionType.DEBIT, 800.0);
        transaction4 = new Transaction("four",payee4, TransactionType.CREDIT, 950.0);
    }

    @org.junit.After
    public void tearDown() throws Exception {
    }

    @Test
    public void test1AddAndSize(){
        checkbook.add(transaction);
        int expected = 1;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2AddAndSize(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        int expected = 2;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3AddAndSize(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        int expected = 3;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test1IsEmpty(){
        checkbook.transactions.add(transaction3);
        Assert.assertFalse(checkbook.isEmpty());
    }

    @Test
    public void test2IsEmpty(){
        checkbook.transactions.clear();
        Assert.assertTrue(checkbook.isEmpty());
    }

    @Test
    public void test3IsEmpty(){
        checkbook.transactions.add(transaction);
        checkbook.transactions.clear();
        Assert.assertTrue(checkbook.isEmpty());
    }

    @Test
    public void test1RemoveAndContains(){
        checkbook.transactions.add(transaction3);
        Assert.assertTrue(checkbook.contains(transaction3));
        checkbook.remove(transaction3);
        Assert.assertFalse(checkbook.contains(transaction3));
    }

    @Test
    public void test2RemoveAndContains(){
        checkbook.transactions.add(transaction2);
        Assert.assertTrue(checkbook.contains(transaction2));
        checkbook.remove(transaction2);
        Assert.assertFalse(checkbook.contains(transaction2));
    }

    @Test
    public void test3RemoveAndContains(){
        checkbook.transactions.add(transaction);
        Assert.assertTrue(checkbook.contains(transaction));
        checkbook.remove(transaction);
        Assert.assertFalse(checkbook.contains(transaction));
    }

    @Test
    public void test1Iterator(){
        checkbook.transactions.add(transaction);
        Iterator iterator = checkbook.iterator();
        Assert.assertTrue(iterator.hasNext());
    }

    @Test
    public void test2Iterator(){
        checkbook.transactions.clear();
        Iterator iterator = checkbook.iterator();
        Assert.assertFalse(iterator.hasNext());
    }

    @Test
    public void test3Iterator(){
        checkbook.transactions.add(transaction3);
        Iterator iterator = checkbook.iterator();
        Assert.assertTrue(iterator.hasNext());
    }

    @Test
    public void test1ToArray(){
        checkbook.add(transaction);
        Transaction[] expected = {transaction};
        Transaction[] actual = checkbook.toArray();
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
    }

    @Test
    public void test2ToArray(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        Transaction[] expected = {transaction,transaction2};
        Transaction[] actual = checkbook.toArray();
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
    }

    @Test
    public void test3ToArray(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        Transaction[] expected = {transaction, transaction2, transaction3};
        Transaction[] actual = checkbook.toArray();
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
        Assert.assertEquals(expected[2], actual[2]);

    }

    @Test
    public void test1ToArray2(){
        checkbook.add(transaction);
        Transaction[] transactions = {transaction2};
        Transaction[] expected = {transaction, transaction2};
        Transaction[] actual = checkbook.toArray(transactions);
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
    }

    @Test
    public void test2ToArray2(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        Transaction[] transactions = {transaction3};
        Transaction[] expected = {transaction,transaction2, transaction3};
        Transaction[] actual = checkbook.toArray(transactions);
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
        Assert.assertEquals(expected[2], actual[2]);
    }

    @Test
    public void test3ToArray2(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        Transaction[] transactions = {transaction4};
        Transaction[] expected = {transaction, transaction2, transaction3, transaction4};
        Transaction[] actual = checkbook.toArray(transactions);
        Assert.assertEquals(expected.length, actual.length);
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
        Assert.assertEquals(expected[2], actual[2]);
        Assert.assertEquals(expected[3], actual[3]);
    }

    @Test
    public void test1CheckBookValue(){
        checkbook.add(transaction);
        double expected = 200.0;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual, 0.0);
    }

    @Test
    public void test2CheckBookValue(){
        checkbook.transactions.clear();
        checkbook.add(transaction);
        checkbook.add(transaction);
        double expected = 400.0;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual, 0.0);
    }

    @Test
    public void test3CheckBookValue(){
        checkbook.add(transaction);
        checkbook.add(transaction3);
        double expected = 1000.0;
        double actual = checkbook.checkbookValue();
        Assert.assertEquals(expected, actual, 0.0);
    }

    @Test
    public void test1GetTransactionsForDate(){
        checkbook.add(transaction);
        Transaction[] expected = {transaction};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test2GetTransactionsForDate(){
        checkbook.transactions.clear();
        checkbook.add(transaction);
        checkbook.add(transaction2);
        Transaction[] expected = {transaction, transaction2};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test3GetTransactionsForDate(){
        checkbook.transactions.clear();
        checkbook.add(transaction);
        Transaction[] expected = {transaction};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test1GetAllTransactionsForPayee(){
        payee1.setGivenName("thulasi");
        payee1.setFamilyName("puppala");
        payee1.setEmailAddress("thulasi@gmail.com");
        checkbook.add(transaction);
        Transaction[] expected = {transaction};
        Transaction[] actual = checkbook.getAllTransactionsForPayee("thulasi");
        Assert.assertEquals(expected[0], actual[0]);

    }

    @Test
    public void test2GetAllTransactionsForPayee(){
        payee2.setGivenName("sreeram");
        payee2.setFamilyName("jonnageddala");
        payee2.setEmailAddress("sreeram@gmail.com");
        checkbook.add(transaction2);
        Transaction[] expected = {transaction2};
        Transaction[] actual = checkbook.getAllTransactionsForPayee("sreeram");
        Assert.assertEquals(expected[0], actual[0]);

    }

    @Test
    public void test3GetAllTransactionsForPayee(){
        payee3.setGivenName("sreya");
        payee3.setFamilyName("jonnageddala");
        payee3.setEmailAddress("sreya@gmail.com");
        checkbook.add(transaction3);
        Transaction[] expected = {transaction3};
        Transaction[] actual = checkbook.getAllTransactionsForPayee("sreya");
        Assert.assertEquals(expected[0], actual[0]);

    }

    @Test
    public void test1GetAllDebitTransactions(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        Transaction[] expected = {transaction, transaction3};
        Transaction[] actual = checkbook.getAllDebitTransactions();
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
    }

    @Test
    public void test2GetAllDebitTransactions(){
        checkbook.add(transaction);
        Transaction[] expected = {transaction};
        Transaction[] actual = checkbook.getAllDebitTransactions();
        Assert.assertEquals(expected[0], actual[0]);
    }

    @Test
    public void test3GetAllDebitTransactions(){
        checkbook.transactions.clear();
        Transaction[] expected = {};
        Transaction[] actual = checkbook.getAllDebitTransactions();
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test1GetAllCreditTransactions(){
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        Transaction[] expected = {transaction2, transaction4};
        Transaction[] actual = checkbook.getAllCreditTransactions();
        Assert.assertEquals(expected[0], actual[0]);
        Assert.assertEquals(expected[1], actual[1]);
    }

    @Test
    public void test2GetAllCreditTransactions(){
        checkbook.transactions.clear();
        checkbook.add(transaction);
        Transaction[] expected = {};
        Transaction[] actual = checkbook.getAllCreditTransactions();
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test3GetAllCreditTransactions(){
        checkbook.transactions.clear();
        Transaction[] expected = {};
        Transaction[] actual = checkbook.getAllCreditTransactions();
        Assert.assertEquals(expected.length, actual.length);
    }

    @Test
    public void test1ContainsAll(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        boolean actual = checkbook.containsAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test2ContainsAll(){
        checkbook.add(transaction2);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        boolean actual = checkbook.containsAll(transactions);
        Assert.assertFalse(actual);
    }

    @Test
    public void test3ContainsAll(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        transactions.add(transaction3);
        boolean actual = checkbook.containsAll(transactions);
        Assert.assertTrue(actual);
    }


    @Test
    public void test1AddAll(){
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        boolean actual = checkbook.addAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test2AddAll(){
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        boolean actual = checkbook.addAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test3AddAll(){
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        transactions.add(transaction3);
        boolean actual = checkbook.addAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test1removeAll(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction);
        transactions.add(transaction2);
        boolean actual = checkbook.removeAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test2removeAll(){
        checkbook.add(transaction);
        checkbook.add(transaction2);
        checkbook.add(transaction);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction3);
        transactions.add(transaction2);
        boolean actual = checkbook.removeAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test3removeAll(){
        checkbook.add(transaction2);
        checkbook.add(transaction4);
        List<Transaction> transactions = new ArrayList<>();
        transactions.add(transaction2);
        transactions.add(transaction4);
        boolean actual = checkbook.removeAll(transactions);
        Assert.assertTrue(actual);
    }

    @Test
    public void test1Clear(){
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.clear();
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test2Clear(){
        checkbook.add(transaction4);
        checkbook.clear();
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void test3Clear(){
        checkbook.clear();
        int expected = 0;
        int actual = checkbook.size();
        Assert.assertEquals(expected, actual);
    }
}